@@ -35,6 +35,7 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
 #include "gsd-osd-window.h"
 
@@ -55,7 +56,7 @@ struct GsdOsdWindowPrivate
 };
 
 enum {
-        EXPOSE_WHEN_COMPOSITED,
+        DRAW_WHEN_COMPOSITED,
         LAST_SIGNAL
 };
 
@@ -191,9 +192,192 @@ gsd_osd_window_draw_rounded_rectangle (cairo_t* cr,
         cairo_close_path (cr);
 }
 
+static void
+rgb_to_hls (gdouble *r,
+            gdouble *g,
+            gdouble *b)
+{
+        gdouble min;
+        gdouble max;
+        gdouble red;
+        gdouble green;
+        gdouble blue;
+        gdouble h, l, s;
+        gdouble delta;
+
+        red = *r;
+        green = *g;
+        blue = *b;
+
+        if (red > green)
+        {
+                if (red > blue)
+                        max = red;
+                else
+                        max = blue;
+
+                if (green < blue)
+                        min = green;
+                else
+                        min = blue;
+        }
+        else
+        {
+                if (green > blue)
+                        max = green;
+                else
+                        max = blue;
+
+                if (red < blue)
+                        min = red;
+                else
+                        min = blue;
+        }
+
+        l = (max + min) / 2;
+        s = 0;
+        h = 0;
+
+        if (max != min)
+        {
+                if (l <= 0.5)
+                        s = (max - min) / (max + min);
+                else
+                        s = (max - min) / (2 - max - min);
+
+                delta = max -min;
+                if (red == max)
+                        h = (green - blue) / delta;
+                else if (green == max)
+                        h = 2 + (blue - red) / delta;
+                else if (blue == max)
+                        h = 4 + (red - green) / delta;
+
+                h *= 60;
+                if (h < 0.0)
+                        h += 360;
+        }
+
+        *r = h;
+        *g = l;
+        *b = s;
+}
+
+static void
+hls_to_rgb (gdouble *h,
+            gdouble *l,
+            gdouble *s)
+{
+        gdouble hue;
+        gdouble lightness;
+        gdouble saturation;
+        gdouble m1, m2;
+        gdouble r, g, b;
+
+        lightness = *l;
+        saturation = *s;
+
+        if (lightness <= 0.5)
+                m2 = lightness * (1 + saturation);
+        else
+                m2 = lightness + saturation - lightness * saturation;
+        m1 = 2 * lightness - m2;
+
+        if (saturation == 0)
+        {
+                *h = lightness;
+                *l = lightness;
+                *s = lightness;
+        }
+        else
+        {
+                hue = *h + 120;
+                while (hue > 360)
+                        hue -= 360;
+                while (hue < 0)
+                        hue += 360;
+
+                if (hue < 60)
+                        r = m1 + (m2 - m1) * hue / 60;
+                else if (hue < 180)
+                        r = m2;
+                else if (hue < 240)
+                        r = m1 + (m2 - m1) * (240 - hue) / 60;
+                else
+                        r = m1;
+
+                hue = *h;
+                while (hue > 360)
+                        hue -= 360;
+                while (hue < 0)
+                        hue += 360;
+
+                if (hue < 60)
+                        g = m1 + (m2 - m1) * hue / 60;
+                else if (hue < 180)
+                        g = m2;
+                else if (hue < 240)
+                        g = m1 + (m2 - m1) * (240 - hue) / 60;
+                else
+                        g = m1;
+
+                hue = *h - 120;
+                while (hue > 360)
+                        hue -= 360;
+                while (hue < 0)
+                        hue += 360;
+
+                if (hue < 60)
+                        b = m1 + (m2 - m1) * hue / 60;
+                else if (hue < 180)
+                        b = m2;
+                else if (hue < 240)
+                        b = m1 + (m2 - m1) * (240 - hue) / 60;
+                else
+                        b = m1;
+
+                *h = r;
+                *l = g;
+                *s = b;
+        }
+}
+
 void
-gsd_osd_window_color_reverse (const GdkColor *a,
-                              GdkColor       *b)
+gsd_osd_window_color_shade (GdkRGBA *a,
+                            gdouble   k)
+{
+        gdouble red;
+        gdouble green;
+        gdouble blue;
+
+        red = a->red;
+        green = a->green;
+        blue = a->blue;
+
+        rgb_to_hls (&red, &green, &blue);
+
+        green *= k;
+        if (green > 1.0)
+                green = 1.0;
+        else if (green < 0.0)
+                green = 0.0;
+
+        blue *= k;
+        if (blue > 1.0)
+                blue = 1.0;
+        else if (blue < 0.0)
+                blue = 0.0;
+
+        hls_to_rgb (&red, &green, &blue);
+
+        a->red = red;
+        a->green = green;
+        a->blue = blue;
+}
+
+
+void
+gsd_osd_window_color_reverse (GdkRGBA *a)
 {
         gdouble red;
         gdouble green;
@@ -202,9 +386,9 @@ gsd_osd_window_color_reverse (const GdkColor *a,
         gdouble s;
         gdouble v;
 
-        red = (gdouble) a->red / 65535.0;
-        green = (gdouble) a->green / 65535.0;
-        blue = (gdouble) a->blue / 65535.0;
+        red = a->red;
+        green = a->green;
+        blue = a->blue;
 
         gtk_rgb_to_hsv (red, green, blue, &h, &s, &v);
 
@@ -216,37 +400,33 @@ gsd_osd_window_color_reverse (const GdkColor *a,
 
         gtk_hsv_to_rgb (h, s, v, &red, &green, &blue);
 
-        b->red = red * 65535.0;
-        b->green = green * 65535.0;
-        b->blue = blue * 65535.0;
+        a->red = red;
+        a->green = green;
+        a->blue = blue;
 }
 
-/* This is our expose-event handler when the window is in a compositing manager.
+/* This is our draw handler when the window is in a compositing manager.
  * We draw everything by hand, using Cairo, so that we can have a nice
  * transparent/rounded look.
  */
 static void
-expose_when_composited (GtkWidget *widget, GdkEventExpose *event)
+draw_when_composited (GtkWidget *widget, cairo_t *orig_cr)
 {
-	GsdOsdWindow    *window;
-        cairo_t         *context;
+        GsdOsdWindow    *window;
         cairo_t         *cr;
         cairo_surface_t *surface;
         int              width;
         int              height;
-        GtkStyle        *style;
-        GdkColor         color;
-        double           r, g, b;
+        GtkStyleContext *context;
+        GdkRGBA          acolor;
 
-	window = GSD_OSD_WINDOW (widget);
+        window = GSD_OSD_WINDOW (widget);
 
-        context = gdk_cairo_create (gtk_widget_get_window (widget));
-
-        style = gtk_widget_get_style (widget);
-        cairo_set_operator (context, CAIRO_OPERATOR_SOURCE);
+        context = gtk_widget_get_style_context (widget);
+        cairo_set_operator (orig_cr, CAIRO_OPERATOR_SOURCE);
         gtk_window_get_size (GTK_WINDOW (widget), &width, &height);
 
-        surface = cairo_surface_create_similar (cairo_get_target (context),
+        surface = cairo_surface_create_similar (cairo_get_target (orig_cr),
                                                 CAIRO_CONTENT_COLOR_ALPHA,
                                                 width,
                                                 height);
@@ -265,86 +445,84 @@ expose_when_composited (GtkWidget *widget, GdkEventExpose *event)
 
         /* draw a box */
         gsd_osd_window_draw_rounded_rectangle (cr, 1.0, 0.5, 0.5, height / 10, width-1, height-1);
-        gsd_osd_window_color_reverse (&style->bg[GTK_STATE_NORMAL], &color);
-        r = (float)color.red / 65535.0;
-        g = (float)color.green / 65535.0;
-        b = (float)color.blue / 65535.0;
-        cairo_set_source_rgba (cr, r, g, b, BG_ALPHA);
+        gtk_style_context_get_background_color (context, GTK_STATE_NORMAL, &acolor);
+        gsd_osd_window_color_reverse (&acolor);
+        acolor.alpha = BG_ALPHA;
+        gdk_cairo_set_source_rgba (cr, &acolor);
         cairo_fill_preserve (cr);
 
-        gsd_osd_window_color_reverse (&style->text_aa[GTK_STATE_NORMAL], &color);
-        r = (float)color.red / 65535.0;
-        g = (float)color.green / 65535.0;
-        b = (float)color.blue / 65535.0;
-        cairo_set_source_rgba (cr, r, g, b, BG_ALPHA / 2);
+        /* FIXME use &style->text_aa[GTK_STATE_NORMAL] instead? */
+        gtk_style_context_get_color (context, GTK_STATE_NORMAL, &acolor);
+        gsd_osd_window_color_reverse (&acolor);
+        acolor.alpha = BG_ALPHA / 2;
+        gdk_cairo_set_source_rgba (cr, &acolor);
         cairo_set_line_width (cr, 1);
         cairo_stroke (cr);
 
-        g_signal_emit (window, signals[EXPOSE_WHEN_COMPOSITED], 0, cr);
+        g_signal_emit (window, signals[DRAW_WHEN_COMPOSITED], 0, cr);
 
         cairo_destroy (cr);
 
         /* Make sure we have a transparent background */
-        cairo_rectangle (context, 0, 0, width, height);
-        cairo_set_source_rgba (context, 0.0, 0.0, 0.0, 0.0);
-        cairo_fill (context);
+        cairo_rectangle (orig_cr, 0, 0, width, height);
+        cairo_set_source_rgba (orig_cr, 0.0, 0.0, 0.0, 0.0);
+        cairo_fill (orig_cr);
 
-        cairo_set_source_surface (context, surface, 0, 0);
-        cairo_paint_with_alpha (context, window->priv->fade_out_alpha);
+        cairo_set_source_surface (orig_cr, surface, 0, 0);
+        cairo_paint_with_alpha (orig_cr, window->priv->fade_out_alpha);
 
  done:
         if (surface != NULL) {
                 cairo_surface_destroy (surface);
         }
-        cairo_destroy (context);
 }
 
-/* This is our expose-event handler when the window is *not* in a compositing manager.
+/* This is our draw handler when the window is *not* in a compositing manager.
  * We just draw a rectangular frame by hand.  We do this with hardcoded drawing code,
  * instead of GtkFrame, to avoid changing the window's internal widget hierarchy:  in
  * either case (composited or non-composited), callers can assume that this works
  * identically to a GtkWindow without any intermediate widgetry.
  */
 static void
-expose_when_not_composited (GtkWidget *widget, GdkEventExpose *event)
+draw_when_not_composited (GtkWidget *widget, cairo_t *cr)
 {
-	GsdOsdWindow *window;
-	GtkAllocation allocation;
+        GsdOsdWindow *window;
+        GtkStyleContext *context;
+        int width;
+        int height;
 
-	window = GSD_OSD_WINDOW (widget);
+        window = GSD_OSD_WINDOW (widget);
 
-	gtk_widget_get_allocation (widget, &allocation);
+        width = gtk_widget_get_allocated_width (widget);
+        height = gtk_widget_get_allocated_width (widget);
+        context = gtk_widget_get_style_context (widget);
 
-	gtk_paint_shadow (gtk_widget_get_style (widget),
-			  gtk_widget_get_window (widget),
-			  gtk_widget_get_state (widget),
-			  GTK_SHADOW_OUT,
-			  &event->area,
-			  widget,
-			  NULL, /* NULL detail -> themes should use the GsdOsdWindow widget name, probably */
-			  0,
-			  0,
-			  allocation.width,
-			  allocation.height);
+        gtk_style_context_set_state (context, GTK_STATE_FLAG_ACTIVE);
+        gtk_render_frame (context,
+                          cr,
+                          0,
+                          0,
+                          width,
+                          height);
 }
 
 static gboolean
-gsd_osd_window_expose_event (GtkWidget          *widget,
-			     GdkEventExpose     *event)
+gsd_osd_window_draw (GtkWidget *widget,
+                     cairo_t   *cr)
 {
-	GsdOsdWindow *window;
-	GtkWidget *child;
+        GsdOsdWindow *window;
+        GtkWidget *child;
 
-	window = GSD_OSD_WINDOW (widget);
+        window = GSD_OSD_WINDOW (widget);
 
-	if (window->priv->is_composited)
-		expose_when_composited (widget, event);
-	else
-		expose_when_not_composited (widget, event);
+        if (window->priv->is_composited)
+                draw_when_composited (widget, cr);
+        else
+                draw_when_not_composited (widget, cr);
 
-	child = gtk_bin_get_child (GTK_BIN (window));
-	if (child)
-		gtk_container_propagate_expose (GTK_CONTAINER (window), child, event);
+        child = gtk_bin_get_child (GTK_BIN (window));
+        if (child)
+                gtk_container_propagate_draw (GTK_CONTAINER (window), child, cr);
 
         return FALSE;
 }
@@ -379,69 +557,82 @@ gsd_osd_window_real_hide (GtkWidget *widget)
 static void
 gsd_osd_window_real_realize (GtkWidget *widget)
 {
-        GdkColormap *colormap;
-        GtkAllocation allocation;
-        GdkBitmap *mask;
-        cairo_t *cr;
+        cairo_region_t *region;
+        GdkScreen *screen;
+        GdkVisual *visual;
 
-        colormap = gdk_screen_get_rgba_colormap (gtk_widget_get_screen (widget));
-
-        if (colormap != NULL) {
-                gtk_widget_set_colormap (widget, colormap);
+        screen = gtk_widget_get_screen (widget);
+        visual = gdk_screen_get_rgba_visual (screen);
+        if (visual == NULL) {
+                visual = gdk_screen_get_system_visual (screen);
         }
 
+        gtk_widget_set_visual (widget, visual);
+
         if (GTK_WIDGET_CLASS (gsd_osd_window_parent_class)->realize) {
                 GTK_WIDGET_CLASS (gsd_osd_window_parent_class)->realize (widget);
         }
 
-        gtk_widget_get_allocation (widget, &allocation);
-        mask = gdk_pixmap_new (gtk_widget_get_window (widget),
-                               allocation.width,
-                               allocation.height,
-                               1);
-        cr = gdk_cairo_create (mask);
-
-        cairo_set_source_rgba (cr, 1., 1., 1., 0.);
-        cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
-        cairo_paint (cr);
-
         /* make the whole window ignore events */
-        gdk_window_input_shape_combine_mask (gtk_widget_get_window (widget), mask, 0, 0);
-        g_object_unref (mask);
-        cairo_destroy (cr);
+        region = cairo_region_create ();
+        gtk_widget_input_shape_combine_region (widget, region);
+        cairo_region_destroy (region);
 }
 
 static void
-gsd_osd_window_style_set (GtkWidget *widget,
-                          GtkStyle  *previous_style)
+gsd_osd_window_style_updated (GtkWidget *widget)
 {
-        GtkStyle *style;
+        GtkStyleContext *context;
+        GtkBorder padding;
 
-        GTK_WIDGET_CLASS (gsd_osd_window_parent_class)->style_set (widget, previous_style);
+        GTK_WIDGET_CLASS (gsd_osd_window_parent_class)->style_updated (widget);
 
         /* We set our border width to 12 (per the GNOME standard), plus the
-         * thickness of the frame that we draw in our expose handler.  This will
+         * thickness of the frame that we draw in our draw handler.  This will
          * make our child be 12 pixels away from the frame.
          */
 
-        style = gtk_widget_get_style (widget);
-        gtk_container_set_border_width (GTK_CONTAINER (widget), 12 + MAX (style->xthickness, style->ythickness));
+        context = gtk_widget_get_style_context (widget);
+        gtk_style_context_get_padding (context, GTK_STATE_NORMAL, &padding);
+        gtk_container_set_border_width (GTK_CONTAINER (widget), 12 + MAX (padding.left, padding.top));
 }
 
 static void
-gsd_osd_window_size_request (GtkWidget      *widget,
-                             GtkRequisition *requisition)
+gsd_osd_window_get_preferred_width (GtkWidget *widget,
+                                    gint      *minimum,
+                                    gint      *natural)
 {
-        GtkStyle *style;
+        GtkStyleContext *context;
+        GtkBorder padding;
 
-        GTK_WIDGET_CLASS (gsd_osd_window_parent_class)->size_request (widget, requisition);
+        GTK_WIDGET_CLASS (gsd_osd_window_parent_class)->get_preferred_width (widget, minimum, natural);
 
-        /* See the comment in gsd_osd_window_style_set() for why we add the thickness here */
+        /* See the comment in gsd_osd_window_style_updated() for why we add the padding here */
 
-        style = gtk_widget_get_style (widget);
+        context = gtk_widget_get_style_context (widget);
+        gtk_style_context_get_padding (context, GTK_STATE_NORMAL, &padding);
 
-        requisition->width  += style->xthickness;
-        requisition->height += style->ythickness;
+        *minimum += padding.left;
+        *natural += padding.left;
+}
+
+static void
+gsd_osd_window_get_preferred_height (GtkWidget *widget,
+                                     gint      *minimum,
+                                     gint      *natural)
+{
+        GtkStyleContext *context;
+        GtkBorder padding;
+
+        GTK_WIDGET_CLASS (gsd_osd_window_parent_class)->get_preferred_height (widget, minimum, natural);
+
+        /* See the comment in gsd_osd_window_style_updated() for why we add the padding here */
+
+        context = gtk_widget_get_style_context (widget);
+        gtk_style_context_get_padding (context, GTK_STATE_NORMAL, &padding);
+
+        *minimum += padding.top;
+        *natural += padding.top;
 }
 
 static GObject *
@@ -475,18 +666,19 @@ gsd_osd_window_class_init (GsdOsdWindowClass *klass)
         widget_class->show = gsd_osd_window_real_show;
         widget_class->hide = gsd_osd_window_real_hide;
         widget_class->realize = gsd_osd_window_real_realize;
-        widget_class->style_set = gsd_osd_window_style_set;
-        widget_class->size_request = gsd_osd_window_size_request;
-	widget_class->expose_event = gsd_osd_window_expose_event;
+        widget_class->style_updated = gsd_osd_window_style_updated;
+        widget_class->get_preferred_width = gsd_osd_window_get_preferred_width;
+        widget_class->get_preferred_height = gsd_osd_window_get_preferred_height;
+        widget_class->draw = gsd_osd_window_draw;
 
-        signals[EXPOSE_WHEN_COMPOSITED] = g_signal_new ("expose-when-composited",
-                                                        G_TYPE_FROM_CLASS (gobject_class),
-                                                        G_SIGNAL_RUN_FIRST,
-                                                        G_STRUCT_OFFSET (GsdOsdWindowClass, expose_when_composited),
-                                                        NULL, NULL,
-                                                        g_cclosure_marshal_VOID__POINTER,
-                                                        G_TYPE_NONE, 1,
-                                                        G_TYPE_POINTER);
+        signals[DRAW_WHEN_COMPOSITED] = g_signal_new ("draw-when-composited",
+                                                      G_TYPE_FROM_CLASS (gobject_class),
+                                                      G_SIGNAL_RUN_FIRST,
+                                                      G_STRUCT_OFFSET (GsdOsdWindowClass, draw_when_composited),
+                                                      NULL, NULL,
+                                                      g_cclosure_marshal_VOID__POINTER,
+                                                      G_TYPE_NONE, 1,
+                                                      G_TYPE_POINTER);
 
         g_type_class_add_private (klass, sizeof (GsdOsdWindowPrivate));
 }
@@ -545,7 +737,7 @@ gsd_osd_window_init (GsdOsdWindow *window)
 
                 window->priv->fade_out_alpha = 1.0;
         } else {
-		gtk_container_set_border_width (GTK_CONTAINER (window), 12);
+                gtk_container_set_border_width (GTK_CONTAINER (window), 12);
         }
 }
 
